unit dbstru;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FileCtrl, StdCtrls, DB, DBTables, ExtCtrls;

type
  Tfdbstruct = class(TForm)
    M: TMemo;
    T: TTable;
    Panel1: TPanel;
    buDump: TButton;
    DriveComboBox1: TDriveComboBox;
    DL: TDirectoryListBox;
    FL: TFileListBox;
    SD: TSaveDialog;
    procedure FLClick(Sender: TObject);
    procedure MDblClick(Sender: TObject);
    procedure MClick(Sender: TObject);
    procedure buDumpClick(Sender: TObject);
  private
    procedure ListTable(const TName:string);
    { Private declarations }
  public
    { Public declarations }
  end;

Function Substr(const S,O,R:String):String;

var
  fdbstruct: Tfdbstruct;

implementation

{$R *.DFM}

procedure Tfdbstruct.ListTable(const TName:string);
var
 i : integer;
 iname,ifields,ioptions,S : String;
 fname,ftype,frequired,fsize:string;
begin
  M.clear;

  with T do begin
    if active then close;
    Tablename := DL.directory+'\'+TName;
    try
      open;
    except
      beep;
      showmessage('Unable to open table '+TName);
    end;
    if not active then exit;

    S := '';
    for i := 1 to length(Tname) do begin
      if copy(Tname,i,1)='.' then break;
      S := S + copy(Tname,i,1)+' ';
    end;
    S := uppercase(S);

    M.lines.append(format('TABLE: %s         %s',[S,Tablename]));
    M.lines.append('------------------------------------------------------------------');

    M.lines.append('FIELDS');
    M.lines.append(format('    %-20s %-21s %5s %s  ',['Name','Type','Size','Options']));
    M.lines.append(format('    %-20s %-21s %5s %s  ',['----','----','----','-------']));

    for i := 0 to Fieldcount -1 do begin
      with fieldDefs.items[i] do begin
        fname := name;
        if required then frequired := 'Required' else frequired := 'Optional';
        case DataType of
        ftString     : ftype := 'Character/string';
        ftSmallint   : ftype := '16-bit int';
        ftInteger    : ftype := '32-bit int';
        ftWord	     : ftype := '16-bit unsigned';
        ftBoolean    : ftype := 'Boolean';
        ftFloat	     : ftype := 'Floating-point';
        ftCurrency   : ftype := 'Money';
        ftBCD        : ftype := 'BCD';
        ftDate       : ftype := 'Date';
        ftTime       : ftype := 'Time';
        ftDateTime   : ftype := 'Date and time';
        ftBytes	     : ftype := 'Fixed no of bytes';
        ftVarBytes   : ftype := 'Variable no. of bytes';
        ftAutoInc    : begin
                         ftype := 'Auto-inc counter';
                         frequired := 'Automatic';
                       end;
        ftBlob       : ftype := 'Binary Large OBject';
        ftMemo       : ftype := 'Text memo';
        ftGraphic    : ftype := 'Bitmap';
        ftFmtMemo    : ftype := 'Formatted text memo';
        ftParadoxOle : ftype := 'Paradox OLE';
        ftDBaseOle   : ftype := 'dBASE OLE';
        ftTypedBinary: ftype := 'Typed binary';
        else
          ftype := 'Unknown';
        end;{case}

        if datatype in [ftString, ftBCD, ftBytes, ftVarBytes, ftBlob, ftMemo, ftGraphic] then begin
          fsize := inttostr(size);
        end else begin
          fsize := '';
        end;
        M.lines.append(format('    %-20s %-21s %5s %s  ',[fname,ftype,fsize,frequired]));
      end;
    end;

    application.processmessages;

    with IndexDefs do begin
      update;
      if Indexdefs.Count>0 then begin
        M.lines.append('INDEXES');
        M.lines.append(format ('    %-20s %-35S %s',['Name of index','Component fields','Options']));
        M.lines.append(format ('    %-20s %-35S %s',['-------------','----------------','-------']));
      end;
      for i := 0 to Indexdefs.Count -1 do begin
        with items[i] do begin
          INAME := trim(name);
          if iname = '' then iname := '<unnamed>';
          IFIELDS := SubStr(fields,';',',');
          IOPTIONS := '';
          if ixPrimary in options then ioptions := ioptions + 'Primary';
          if ixUnique in options then ioptions := ioptions + ',Unique';
          if ixDescending in options then ioptions := ioptions + ',Descending';
          if ixCaseInsensitive in options then ioptions := ioptions + ',Case insensitive';
          if ioptions<>'' then begin
            if copy(ioptions,1,1)=',' then ioptions := copy(ioptions,2,99);
          end;
          M.lines.append(format ('    %-20s %-35S %s',[Iname,ifields,ioptions]));
        end;
      end;
    end;
  end;
end;

Function Substr(const S,O,R:String):String;
var
  safety,p : integer;
begin
  Result := S;
  safety := 0;
  while safety < 50 do begin     // trap recursive substitutions crudely
    p := pos(O,result);
    if p = 0 then break;
    delete(result,p,length(O));
    if R<>'' then insert(R,Result,p);
    inc(safety);
  end;
end;



procedure Tfdbstruct.FLClick(Sender: TObject);
begin
  M.wordwrap := true;
  listtable(FL.items[fl.itemindex]);
end;

procedure Tfdbstruct.MDblClick(Sender: TObject);
begin
  M.font.size := M.font.size + 2;
end;

procedure Tfdbstruct.MClick(Sender: TObject);
begin
  M.font.size := M.font.size - 1;
end;

procedure Tfdbstruct.buDumpClick(Sender: TObject);
var
  Fout : Textfile;
  i,j : integer;
begin

  if FL.items.count < 1 then begin
    beep;
    showmessage('No tables in this directory!');
    exit;
  end;

  with SD do begin
    initialdir := DL.Directory;
    Filename := 'dbstruct.txt';
    if not execute then exit;
    assignfile(Fout,Filename);
  end;

  rewrite(Fout);
  writeln(Fout,'DATABASE STRUCTURE FOR PARADOX TABLES');
  writeln(Fout,'------------------------------------------------------------------');
  writeln(Fout,'DIRECTORY : '+DL.Directory);
  writeln(Fout,'DATE      : '+formatdatetime('d mmm yy    tt',now));
  writeln(Fout,'------------------------------------------------------------------');

  M.wordwrap := false;

  for i := 0 to FL.items.count -1 do begin
    ListTable(FL.items[i]);
    for j := 0 to M.lines.count-1 do begin
      writeln(Fout,M.lines[j]);
    end;
    writeln(Fout,'');
    writeln(Fout,'');
    writeln(Fout,'------------------------------------------------------------------');
  end;
  closefile(Fout);
end;


end.
